@@ -49,6 +49,7 @@
 #include "nsIDOMText.h"
 #include "nsIThread.h"
 #include "nsIComponentManager.h"
+#include "nsIComponentRegistrar.h"
 #include "nsIGenericFactory.h"
 #include "nsEnigContentHandler.h"
 #include "nsReadableUtils.h"
@@ -106,8 +107,12 @@ nsEnigMimeService::nsEnigMimeService()
 
   if (NS_SUCCEEDED(rv)) {
     // Register factory for dummy handler
-    rv = nsComponentManager::RegisterFactory(info.mCID, info.mDescription,
-                                             info.mContractID, factory, PR_TRUE);
+    nsCOMPtr<nsIComponentRegistrar> registrar;
+    rv = NS_GetComponentRegistrar(getter_AddRefs(registrar));
+    if (NS_FAILED(rv)) return;
+        
+    rv = registrar->RegisterFactory(info.mCID, info.mDescription,
+                                             info.mContractID, factory);
     if (NS_SUCCEEDED(rv)) {
       mDummyHandler = PR_TRUE;
     }
@@ -159,10 +164,15 @@ nsEnigMimeService::Init()
   nsCOMPtr<nsIGenericFactory> factory;
   rv = NS_NewGenericFactory(getter_AddRefs(factory), &info);
   if (NS_FAILED(rv)) return rv;
+  
+  nsCOMPtr<nsIComponentRegistrar> registrar;
+  rv = NS_GetComponentRegistrar(getter_AddRefs(registrar));
+  if (NS_FAILED(rv)) return rv;
 
   // Register factory
-  rv = nsComponentManager::RegisterFactory(info.mCID, info.mDescription,
-                                           info.mContractID, factory, PR_TRUE);
+  rv = registrar->RegisterFactory(info.mCID, info.mDescription,
+                                           info.mContractID, factory);
+
   if (NS_FAILED(rv)) return rv;
 
   DEBUG_LOG(("nsEnigMimeService::Init: registered %s\n", info.mContractID));
