@@ -19,6 +19,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ * Patrick Brunschwig <patrick.brunschwig@gmx.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -40,6 +41,7 @@
 
 #include "nsIEnigMimeService.h"
 #include "nsCOMPtr.h"
+#include "nsVoidArray.h"
 
 class nsEnigMimeService : public nsIEnigMimeService
 {
@@ -57,6 +59,7 @@ public:
 protected:
     PRBool             mDummyHandler;
     PRBool             mInitialized;
+    nsCStringArray     mEncryptedURIs;
 };
 
 #endif
