@@ -74,7 +74,7 @@ static const PRUint32 kCharMax = 1024;
 NS_IMPL_THREADSAFE_ISUPPORTS3(nsEnigMimeVerify,
                               nsIEnigMimeVerify,
                               nsIRequestObserver,
-                              nsIStreamListener);
+                              nsIStreamListener)
 
 // nsEnigMimeVerify implementation
 nsEnigMimeVerify::nsEnigMimeVerify()
@@ -300,6 +300,7 @@ temBoundary += "--";
 
   nsXPIDLString keyId;
   nsXPIDLString userId;
+  nsXPIDLString sigDate;
   nsXPIDLString errorMsg;
 
   nsCOMPtr<nsIEnigmail> enigmailSvc = do_GetService(NS_ENIGMAIL_CONTRACTID, &rv);
@@ -317,6 +318,7 @@ temBoundary += "--";
                                       &statusFlags,
                                       getter_Copies(keyId),
                                       getter_Copies(userId),
+                                      getter_Copies(sigDate),
                                       getter_Copies(errorMsg),
                                       &exitCode);
   if (NS_FAILED(rv)) return rv;
@@ -334,7 +336,7 @@ temBoundary += "--";
   if (securityInfo) {
     nsCOMPtr<nsIEnigMimeHeaderSink> enigHeaderSink = do_QueryInterface(securityInfo);
     if (enigHeaderSink) {
-      rv = enigHeaderSink->UpdateSecurityStatus(mURISpec, exitCode, statusFlags, keyId, userId, errorMsg);
+      rv = enigHeaderSink->UpdateSecurityStatus(mURISpec, exitCode, statusFlags, keyId, userId, sigDate, errorMsg);
     }
   }
 
