@@ -71,7 +71,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS5(nsIPCBuffer,
                               nsIPipeListener,
                               nsIIPCBuffer,
                               nsIInputStream,
-                              nsIRunnable);
+                              nsIRunnable)
 
 
 // nsIPCBuffer implementation
