@@ -39,6 +39,7 @@
 #include "nsCRT.h"
 #include "nsReadableUtils.h"
 #include "netCore.h"
+#include "nsEventQueueUtils.h"
 
 #include "nsIServiceManager.h"
 #include "nsIProxyObjectManager.h"
@@ -160,7 +161,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS8(nsPipeTransport,
                               nsIOutputStream,
                               nsIStreamListener,
                               nsIInputStreamCallback,
-                              nsIOutputStreamCallback);
+                              nsIOutputStreamCallback)
 #else // !MOZILLA_VERSION
 // Mods for Mozilla version prior to 1.3b
 NS_IMPL_THREADSAFE_ISUPPORTS8(nsPipeTransport, 
@@ -171,7 +172,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS8(nsPipeTransport,
                               nsIOutputStream,
                               nsIStreamListener,
                               nsIInputStreamObserver,
-                              nsIOutputStreamObserver);
+                              nsIOutputStreamObserver)
 #endif // !MOZILLA_VERSION
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -921,9 +922,16 @@ nsPipeTransport::AsyncRead(nsIStreamListener *listener,
     mInputStream = asyncInputStream;
     mOutputStream = asyncOutputStream;
 
+    nsCOMPtr<nsIEventQueue> eventQ;
+
+    if (!mNoProxy) {
+      rv = NS_GetCurrentEventQ(getter_AddRefs(eventQ));
+      if (NS_FAILED(rv)) return rv;
+    }
+
     // Set input stream observer (using event queue, if need be)
     rv = asyncInputStream->AsyncWait((nsIInputStreamCallback*) this,
-                                      0, 0, mNoProxy ? NULL : NS_CURRENT_EVENTQ);
+                                      0, 0, eventQ);
     if (NS_FAILED(rv)) return rv;
 
 #else // !MOZILLA_VERSION
@@ -1205,6 +1213,105 @@ nsPipeTransport::ExecPrompt(const char* command, const char* prompt,
   return NS_OK;
 }
 
+// Read the next line
+NS_IMETHODIMP
+nsPipeTransport::ReadLine(PRInt32 maxOutputLen,
+                            char* *_retval)
+{
+  nsresult rv;
+
+  DEBUG_LOG(("nsPipeTransport::ReadLine: maxOutputLen=%d\n", maxOutputLen));
+
+  if (!_retval)
+    return NS_ERROR_NULL_POINTER;
+
+  if (!mInputStream) {
+    nsCOMPtr<nsIInputStream> inputStream;
+    rv = OpenInputStream(0, PRUint32(-1), 0, getter_AddRefs(inputStream));
+    if (NS_FAILED(rv)) return rv;
+  }
+
+  if (mStdoutStream != STREAM_SYNC_OPEN)
+    return NS_ERROR_NOT_AVAILABLE;
+
+
+  PRInt32 returnCount = -1;
+
+  if (maxOutputLen != 0) {
+    char buf[kCharMax];
+    PRUint32 readCount, readMax;
+
+    PRUint32 remainingCount = (maxOutputLen > 0) ? maxOutputLen : kCharMax;
+    
+    if (mExecBuf.Length()>0) {
+      mExecBuf.ReplaceSubstring("\r\n", "\n");
+      mExecBuf.ReplaceSubstring("\r", "\n");
+      returnCount = mExecBuf.Find("\n", PR_FALSE, 0);
+      DEBUG_LOG(("nsPipeTransport::ReadLine: returnCount=%d\n", returnCount));
+    }
+    
+    if (returnCount < 0) {
+      while (remainingCount > 0) {
+        readMax = (remainingCount < kCharMax) ? remainingCount : kCharMax;
+  
+        if (mStdoutPoller) {
+          // Fail if poller has been interrupted
+          PRBool interrupted;
+          rv = mStdoutPoller->IsInterrupted(&interrupted);
+          if (NS_FAILED(rv)) return rv;
+      
+          if (interrupted)
+            return NS_BASE_STREAM_CLOSED;
+        }
+        rv = mInputStream->Read((char *) buf, kCharMax, &readCount);
+        if (NS_FAILED(rv)) return rv;
+  
+        if (readCount < 0)
+          return NS_ERROR_FAILURE;
+  
+        if (readCount == 0)
+          break;             // End-of-file
+  
+        mExecBuf.Append(buf, readCount);
+  
+        if (mExecBuf.Length() >= 1) {
+          mExecBuf.ReplaceSubstring("\r\n", "\n");
+          mExecBuf.ReplaceSubstring("\r", "\n");
+          returnCount = mExecBuf.Find("\n", PR_FALSE, 0);
+  
+          if (returnCount >= 0) {
+            break;
+          }
+        }
+  
+        if (maxOutputLen > 0) {
+          // Limited read
+          remainingCount -= readCount;
+        } else {
+          // Unlimited read
+          remainingCount = kCharMax;
+        }
+      }
+    }
+    if (returnCount < 0)
+      returnCount = mExecBuf.Length();  // Return everything
+  }
+  
+  // Duplicate output string and return it
+  nsCAutoString outStr("");
+  if (returnCount >= 0) {
+    mExecBuf.Left(outStr, returnCount);
+    mExecBuf.Cut(0,returnCount+1);
+  }
+  *_retval = nsCRT::strdup(outStr.get());
+  if (!*_retval)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  DEBUG_LOG(("nsPipeTransport::readLine: *_retval='%s'\n", *_retval));
+
+  return NS_OK;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // nsIRequest methods
 ///////////////////////////////////////////////////////////////////////////////
@@ -1470,7 +1577,7 @@ nsPipeTransport::OnDataAvailable(nsIRequest* aRequest, nsISupports* aContext,
 #ifdef MOZILLA_VERSION
 
 ///////////////////////////////////////////////////////////////////////////////
-// nsIInputStreamNotify methods:
+// nsIInputStreamCallback methods:
 // (Should be invoked in the thread creating nsIPipeTransport object,
 //  unless mNoProxy is true.)
 ///////////////////////////////////////////////////////////////////////////////
@@ -1502,9 +1609,15 @@ nsPipeTransport::OnInputStreamReady(nsIAsyncInputStream* inStr)
                                      mInputStream, 0, available);
     if (NS_FAILED(rv)) return rv;
 
+    nsCOMPtr<nsIEventQueue> eventQ;
+
+    if (!mNoProxy) {
+      rv = NS_GetCurrentEventQ(getter_AddRefs(eventQ));
+      if (NS_FAILED(rv)) return rv;
+    }
+
     // Re-set input stream observer (using event queue, if need be)
-    rv = inStr->AsyncWait((nsIInputStreamCallback*) this,
-                          0, 0, mNoProxy ? NULL : NS_CURRENT_EVENTQ);
+    rv = inStr->AsyncWait((nsIInputStreamCallback*) this, 0, 0, eventQ);
     if (NS_FAILED(rv)) return rv;
 
     return rv;
@@ -1514,7 +1627,7 @@ nsPipeTransport::OnInputStreamReady(nsIAsyncInputStream* inStr)
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// nsIOutputStreamNotify methods:
+// nsIOutputStreamCallback methods:
 // (Should be invoked in the thread creating nsIPipeTransport object)
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -1699,7 +1812,7 @@ nsPipeTransport::StopRequest(nsresult aStatus)
 // nsISupports implementation
 NS_IMPL_THREADSAFE_ISUPPORTS2 (nsStdoutPoller,
                                nsIPipeTransportPoller,
-                               nsIRunnable);
+                               nsIRunnable)
 
 
 // nsStdoutPoller implementation
@@ -2379,7 +2492,7 @@ nsStdoutPoller::Run()
 // nsISupports implementation
 NS_IMPL_THREADSAFE_ISUPPORTS2 (nsStdinWriter,
                                nsIPipeTransportWriter,
-                               nsIRunnable);
+                               nsIRunnable)
 
 
 // nsStdinWriter implementation
