@@ -14,6 +14,9 @@ var gIdentityListPopup;
 var gUseForSigning;
 
 var gKeygenRequest;
+var gAllData = "";
+var gGeneratedKey="";
+var gUsedId;
 
 function enigmailKeygenLoad() {
   DEBUG_LOG("enigmailKeygen.js: Load\n");
@@ -41,6 +44,10 @@ function enigmailKeygenLoad() {
   }  
 }
 
+function enigmailOnClose() {
+  enigmailKeygenCancel();
+}
+
 function enigmailKeygenUnload() {
    DEBUG_LOG("enigmailKeygen.js: Unload\n");
 
@@ -79,11 +86,17 @@ function enigmailKeygenTerminate(terminateArg, ipcRequest) {
    window.focus();
 
    var keygenProcess = ipcRequest.pipeTransport;
-
+   var enigmailSvc = GetEnigmailSvc();
+   if (!enigmailSvc) {
+     EnigAlert(EnigGetString("accessError"));
+   }
    if (keygenProcess && !keygenProcess.isAttached) {
      keygenProcess.terminate();
      var exitCode = keygenProcess.exitCode();
      DEBUG_LOG("enigmailKeygenConsole.htm: exitCode = "+exitCode+"\n");
+     if (enigmailSvc) {
+        exitCode = enigmailSvc.fixExitCode(exitCode, 0);
+     }
    }
 
    enigRefreshConsole();
@@ -91,18 +104,24 @@ function enigmailKeygenTerminate(terminateArg, ipcRequest) {
    ipcRequest.close(true);
 
    if (gUseForSigning.checked) {
-      var identityItem = gIdentityList.selectedItem;
-      var email = identityItem.getAttribute("email");
-      var curId = getCurrentIdentity();
+      var curId = gUsedId;
       curId.setBoolAttribute("enablePgp", true);
       curId.setIntAttribute("pgpKeyMode", 1);
-      curId.setCharAttribute("pgpkeyId", email);
+      if (gGeneratedKey) {
+        curId.setCharAttribute("pgpkeyId", "0x"+gGeneratedKey.substr(-8,8));
+      }
+      else {
+        curId.setCharAttribute("pgpkeyId", curId.email);
+      }
 
       enigmailKeygenUpdate(false, true);
 
       EnigSavePrefs();
 
-      EnigAlert(EnigGetString("genComplete",email));
+      if (EnigConfirm(EnigGetString("genComplete", curId.email))) {
+        EnigCreateRevokeCert(gGeneratedKey, curId.email);
+      }
+      
 
    } else {
       EnigAlert(EnigGetString("genCompleteNoSign"));
@@ -110,10 +129,6 @@ function enigmailKeygenTerminate(terminateArg, ipcRequest) {
 
    enigmailKeygenCloseRequest();
 
-   var enigmailSvc = GetEnigmailSvc();
-   if (!enigmailSvc) {
-     EnigAlert(EnigGetString("accessError"));
-   }
    enigmailSvc.invalidateUserIdList();
    window.close();
 }
@@ -186,11 +201,26 @@ function enigmailKeygenStart() {
       comment += "no password";
    }
 
+   var noExpiry = document.getElementById("noExpiry");
+   var expireInput = document.getElementById("expireInput");
+   var timeScale = document.getElementById("timeScale");
+
+   var expiryTime = 0;
+   if (! noExpiry.checked) {
+      expiryTime = Number(expireInput.value) * Number(timeScale.value);
+      if (expiryTime > 36500) {
+        EnigAlert(EnigGetString("expiryTooLong"));
+        return;
+      }
+   }
+   var keySize = Number(document.getElementById("keySize").value);
+
    var curId = getCurrentIdentity();
+   gUsedId = curId;
 
    var userName = curId.fullName;
    var userEmail = curId.email;
-
+   
    if (!userName) {
       EnigAlert(EnigGetString("passUserName"));
       return;
@@ -206,7 +236,6 @@ function enigmailKeygenStart() {
    var confirmMsg = EnigGetString("keyConfirm",idString);
 
    if (!EnigConfirm(confirmMsg)) {
-     window.close();
      return;
    }
 
@@ -218,7 +247,8 @@ function enigmailKeygenStart() {
                                            userName,
                                            comment,
                                            userEmail,
-                                           0,
+                                           expiryTime,
+                                           keySize,
                                            passphrase,
                                            requestObserver);
    } catch (ex) {
@@ -237,7 +267,7 @@ function enigmailKeygenStart() {
 }
 
 function enigRefreshConsole() {
-  //DEBUG_LOG("enigmailKeygen.js: enigRefreshConsole():\n");
+  //DEBUG_LOG("enigmailKeygen.js: enigRefreshConsole:\n");
 
   if (!gKeygenRequest)
     return;
@@ -255,10 +285,17 @@ function enigRefreshConsole() {
 
         var consoleElement = contentFrame.document.getElementById('console');
 
-        consoleElement.firstChild.data = keygenConsole.getData();
-
+        gAllData += keygenConsole.getNewData();
+        var keyCreatedIndex = gAllData.indexOf("[GNUPG:] KEY_CREATED");
+        if (keyCreatedIndex >0) {
+          gGeneratedKey = gAllData.substr(keyCreatedIndex);
+          gGeneratedKey = gGeneratedKey.replace(/(.*\[GNUPG:\] KEY_CREATED . )([a-fA-F0-9]+)([\n\r].*)*/, "$2");
+          gAllData = gAllData.replace(/\[GNUPG:\] KEY_CREATED . [a-fA-F0-9]+[\n\r]/, "");
+        }
+        gAllData = gAllData.replace(/[\r\n]*\[GNUPG:\] GOOD_PASSPHRASE/g, "").replace(/([\r\n]*\[GNUPG:\] PROGRESS primegen )(.)( \d+ \d+)/g, "$2")
+        consoleElement.firstChild.data = gAllData.replace(/(.{70})/g, "$1\n");
         if (!contentFrame.mouseDownState)
-         contentFrame.scrollTo(0,9999);
+          contentFrame.scrollTo(0,9999);
       }
     }
   } catch (ex) {}
@@ -267,14 +304,28 @@ function enigRefreshConsole() {
 function enigmailKeygenCancel() {
    DEBUG_LOG("enigmailKeygen.js: Cancel\n");
 
-   var confirmMsg = EnigGetString("keyAbort");
-
-   if (EnigConfirm(confirmMsg)) {
+   if (gKeygenRequest) {
+      var closeWin = EnigConfirm(EnigGetString("keyAbort"));
+   }
+   else {
+      closeWin=true;
+   }
+   if (closeWin) {
      enigmailKeygenCloseRequest();
      window.close();
    }
 }
 
+function onNoExpiry() {
+  var noExpiry = document.getElementById("noExpiry");
+  var expireInput = document.getElementById("expireInput");
+  var timeScale = document.getElementById("timeScale");
+  
+  expireInput.disabled=noExpiry.checked;
+  timeScale.disabled=noExpiry.checked;
+}
+
+
 function queryISupportsArray(supportsArray, iid) {
     var result = new Array;
     for (var i=0; i<supportsArray.Count(); i++) {
@@ -318,7 +369,7 @@ function fillIdentityListPopup()
   for (var i=0; i<identities.length; i++) {
     var identity = identities[i];
 
-    dump("id.valid="+identity.valid+"\n");
+    DEBUG_LOG("id.valid="+identity.valid+"\n");
     if (!identity.valid || !identity.email)
       continue;
 
