@@ -582,7 +582,8 @@ function enigMessageDecrypt(event, isAuto) {
       gEnigNoShowReload = true;
 
       var msgFrame = EnigGetFrame(window, "messagepane");
-      msgFrame.location = "enigmail:dummy";
+      //msgFrame.location = "enigmail:dummy";
+      messenger.loadURL(msgFrame, "enigmail:dummy");
 
     }
     return;
@@ -594,10 +595,11 @@ function enigMessageDecrypt(event, isAuto) {
   // special treatment for embedded signed messages
   if (embeddedSigned) {
     if (contentType.search(/^multipart\/encrypted(;|$)/i) == 0) {
+      var maxSize = EnigGetPref("encapsulatedMimeMaxSize")
       var msgSize=messenger.messageServiceFromURI(msgUriSpec).messageURIToMsgHdr(msgUriSpec).messageSize
-      // workaround for bug 5777: try to verify if msg size < 40000
+      // workaround for bug 5777: try to verify if msg size < maxSize
       // to avoid blocking Mozilla
-      if (msgSize < 40000) tryVerify = true;
+      if (msgSize < maxSize) tryVerify = true;
     }
     if (contentType.search(/^multipart\/mixed(;|$)/i) == 0) {
       tryVerify = true;
@@ -866,7 +868,8 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
                                            false);
     gEnigCreatedURIs.push(uri);
 
-    msgFrame.location = uri;
+    //msgFrame.location=uri;
+    messenger.loadURL(msgFrame, uri);
 
   } catch (ex) {
     // Display plain text with hyperlinks
