@@ -865,10 +865,10 @@ function enigEncryptMsg(msgSendType) {
             if ((recipientsSelectionOption==2) ||
                 ((testStatusFlagsObj.value & nsIEnigmail.INVALID_RECIPIENT) &&
                  (recipientsSelectionOption>0))) {
-  
                 var resultObj = new Object();
                 var inputObj = new Object();
                 inputObj.toAddr = toAddr;
+                inputObj.invalidAddr = enigGetInvalidAddress(testErrorMsgObj.value);
                 inputObj.options = "multisel";
                 if (perRecipientRules<2)
                   inputObj.options += ",rulesOption"
@@ -1310,18 +1310,40 @@ function enigMessageSendCheck() {
 /////////////////////////////////////////////////////////////////////////
 function enigModifyCompFields(msgCompFields) {
 
-  if (EnigGetPref("addHeaders")) {
-    var enigmailHeaders = "X-Enigmail-Version: "+gEnigmailVersion+"\r\n"+
-                          "X-Enigmail-Supports: pgp-inline, pgp-mime\r\n";
+  const ENIG_HEADERMODE_KEYID = 0x01;
+  const ENIG_HEADERMODE_URL   = 0x10;
 
-    msgCompFields.otherRandomHeaders += enigmailHeaders;
+  try {
+    if (gEnigIdentity.getBoolAttribute("enablePgp")) {
+      var enigmailHeaders = "";
+      if (EnigGetPref("addHeaders")) {
+        enigmailHeaders += "X-Enigmail-Version: "+gEnigmailVersion+"\r\n";
+      }
+      var pgpHeader="";
+      var openPgpHeaderMode = gEnigIdentity.getIntAttribute("openPgpHeaderMode");
 
-    DEBUG_LOG("enigmailMsgComposeOverlay.js: enigModifyCompFields: otherRandomHeaders = "+
-             msgCompFields.otherRandomHeaders+"\n");
-  }
-  else {
-    DEBUG_LOG("enigmailMsgComposeOverlay.js: enigModifyCompFields: no headers added\n");
+      if (openPgpHeaderMode > 0) pgpHeader = "OpenPGP: ";
+
+      if (openPgpHeaderMode & ENIG_HEADERMODE_KEYID) {
+          var keyId = gEnigIdentity.getCharAttribute("pgpkeyId");
+          if (keyId.substr(0,2).toLowerCase() == "0x") {
+            pgpHeader += "id="+keyId.substr(2);
+          }
+      }
+      if (openPgpHeaderMode & ENIG_HEADERMODE_URL) {
+        if (pgpHeader.indexOf("=") > 0) pgpHeader += ";\r\n\t";
+        pgpHeader += "url="+gEnigIdentity.getCharAttribute("openPgpUrlName");
+      }
+      if (pgpHeader.length > 0) {
+        enigmailHeaders += pgpHeader + "\r\n";
+      }
+      msgCompFields.otherRandomHeaders += enigmailHeaders;
+    }
   }
+  catch (ex) {}
+
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigModifyCompFields: otherRandomHeaders = "+
+           msgCompFields.otherRandomHeaders+"\n");
 }
 
 
