@@ -68,6 +68,7 @@ function enigStartHeaders()
 
   var enigmailBox = document.getElementById("enigmailBox");
   var statusText = document.getElementById("enigmailStatusText");
+  var statusTextBox = document.getElementById("enigmailStatusTextBox");
 
   if (enigmailBox && !enigmailBox.collapsed) {
     enigmailBox.setAttribute("collapsed", "true");
@@ -78,6 +79,9 @@ function enigStartHeaders()
   if (statusText)
     statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureOk");
 
+  if (statusTextBox)
+    statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelSignatureOk");
+    
   statusText = document.getElementById("collapsedEnigmailStatusText");
 
   if (enigmailBox && !enigmailBox.collapsed) {
@@ -154,7 +158,14 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
   DEBUG_LOG("enigmailMsgHdrViewOverlay.js: enigUpdateHdrIcons: exitCode="+exitCode+", statusFlags="+statusFlags+", keyId="+keyId+", userId="+userId+", "+errorMsg+"\n");
 
   gEnigLastEncryptedURI = GetLoadedMessage();
-  
+  var bodyElement = document.getElementById("messagepanebox");
+/*
+  var bodyElement = msgFrame.document.getElementsByTagName("body")[0];
+  if (! bodyElement) {
+    bodyElement = msgFrame.frames.document.body;
+  }
+*/
+
   if (userId && (userId.indexOf("\n")>=0)) {
     var replaceUid=enigMatchUidToSender(userId);
     errorMsg = errorMsg.replace(userId, replaceUid);
@@ -199,7 +210,18 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
       statusFlags |= nsIEnigmail.DECRYPTION_INCOMPLETE;
   }
 
-   var msgSigned = (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
+  if (! EnigGetPref("displayPartiallySigned")) {
+    if ((statusFlags & (nsIEnigmail.PARTIALLY_PGP))
+        && (statusFlags & (nsIEnigmail.BAD_SIGNATURE))) {
+      statusFlags &= ~(nsIEnigmail.BAD_SIGNATURE | nsIEnigmail.PARTIALLY_PGP);
+      if (statusFlags == 0) {
+        errorMsg="";
+        fullStatusInfo="";
+      }
+    }
+  }
+
+  var msgSigned = (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
             nsIEnigmail.GOOD_SIGNATURE |
             nsIEnigmail.EXPIRED_KEY_SIGNATURE |
             nsIEnigmail.EXPIRED_SIGNATURE |
@@ -211,13 +233,17 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
             nsIEnigmail.DECRYPTION_INCOMPLETE |
             nsIEnigmail.DECRYPTION_FAILED));
 
+  if (msgSigned && (statusFlags & nsIEnigmail.IMPORTED_KEY)) {
+    statusFlags &= (~nsIEnigmail.IMPORTED_KEY);
+  }
+  
   if (((!(statusFlags & (nsIEnigmail.DECRYPTION_INCOMPLETE |
             nsIEnigmail.DECRYPTION_FAILED |
             nsIEnigmail.UNVERIFIED_SIGNATURE |
             nsIEnigmail.BAD_SIGNATURE))) ||
-       (statusFlags & nsIEnigmail.DISPLAY_MESSAGE)) &&
-        !(statusFlags & (nsIEnigmail.UNVERIFIED_SIGNATURE |
-          nsIEnigmail.IMPORTED_KEY))) {
+       (statusFlags & nsIEnigmail.DISPLAY_MESSAGE) &&
+        !(statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE)) &&
+          !(statusFlags & nsIEnigmail.IMPORTED_KEY)) {
     // Normal exit / display message
     statusLine = errorMsg;
     statusInfo = statusLine;
@@ -244,8 +270,8 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
     fullStatusInfo = statusInfo;
 
   } else {
-      if (keyId) {
-        statusInfo = EnigGetString("keyNeeded",keyId);
+    if (keyId) {
+      statusInfo = EnigGetString("keyNeeded",keyId);
   
       if (statusFlags & nsIEnigmail.INLINE_KEY) {
         statusLine = statusInfo + EnigGetString("clickDecrypt");
@@ -291,6 +317,8 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
       statusInfo += "\n\n" + errorMsg;
   
     } else if (statusFlags & nsIEnigmail.IMPORTED_KEY) {
+      statusLine = "";
+      statusInfo = "";
       EnigAlert(errorMsg);
   
     } else {
@@ -318,7 +346,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
 
   if (EnigGetPref("displayPartiallySigned")) {
     if (statusFlags & nsIEnigmail.PARTIALLY_PGP) {
-      if  (msgSigned && msgEncrypted) {
+      if (msgSigned && msgEncrypted) {
         statusLine = EnigGetString("msgPart", EnigGetString("msgSignedAndEnc"));
         statusLine += EnigGetString("clickPenKeyDetails");
       }
@@ -344,6 +372,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
 
   var enigmailBox = document.getElementById("enigmailBox");
   var statusText  = document.getElementById("enigmailStatusText");
+  var statusTextBox = document.getElementById("enigmailStatusTextBox");
   var expStatusText  = document.getElementById("expandedEnigmailStatusText");
 
   if (statusArr.length>0) {
@@ -379,6 +408,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
       gSignedUINode.setAttribute("signed", "notok");
       statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureNotOk");
       expStatusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureNotOk");
+      statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelSignatureNotOk");
       gEnigStatusBar.setAttribute("signed", "notok");
     }
     else if ((statusFlags & nsIEnigmail.GOOD_SIGNATURE) &&
@@ -390,19 +420,37 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
       gSignedUINode.setAttribute("signed", "ok");
       statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureOk");
       expStatusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureOk");
+      statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelSignatureOk");
       gEnigStatusBar.setAttribute("signed", "ok");
+      bodyElement.setAttribute("enigSigned", "ok");
     }
-    else if (statusFlags & (nsIEnigmail.UNVERIFIED_SIGNATURE |
-                       nsIEnigmail.REVOKED_KEY |
+    else if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
+      // Display unverified signature icon
+      gSignedUINode.setAttribute("signed", "unknown");
+      statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureUnknown");
+      expStatusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureUnknown");
+      statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelSignatureUnknown");
+      gEnigStatusBar.setAttribute("signed", "unknown");
+    }
+    else if (statusFlags & (nsIEnigmail.REVOKED_KEY |
                        nsIEnigmail.EXPIRED_KEY_SIGNATURE |
                        nsIEnigmail.EXPIRED_SIGNATURE |
                        nsIEnigmail.GOOD_SIGNATURE)) {
       // Display unverified signature icon
       gSignedUINode.setAttribute("signed", "unknown");
-      statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureUnknown");
-      expStatusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureUnknown");
+      statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureVerified");
+      expStatusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureVerified");
+      statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelSignatureVerified");
       gEnigStatusBar.setAttribute("signed", "unknown");
     }
+    else if (statusFlags & nsIEnigmail.INLINE_KEY) {
+      statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureUnknown");
+      statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelSignatureUnknown");
+    }
+    else {
+      statusText.setAttribute("class", "plain enigmailHeaderNameBox enigmailHeaderBoxLabelNoSignature");
+      statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelNoSignature");
+    }
 
     if (statusFlags & nsIEnigmail.DECRYPTION_OKAY) {
       var enigMimeService = Components.classes[ENIG_ENIGMIMESERVICE_CONTRACTID].getService(Components.interfaces.nsIEnigMimeService);
@@ -420,6 +468,8 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, er
       // Display un-encrypted icon
       gEncryptedUINode.setAttribute("encrypted", "notok");
       gEnigStatusBar.setAttribute("encrypted", "notok");
+      statusText.setAttribute("class", "plain enigmailHeaderValue enigmailHeaderBoxLabelSignatureNotOk");
+      statusTextBox.setAttribute("class", "enigmailHeaderNameBox enigmailHeaderBoxLabelSignatureNotOk");
     }
 
   } catch (ex) {}
@@ -681,10 +731,15 @@ if (messageHeaderSink) {
     ClearAttachmentList();
     ClearEditMessageButton();
     
-    // TB 0.8 and newer
-    if (typeof(SetUpRemoteContentBar) == "function")
+    if (typeof(gMessageNotificationBar) == "object") {
+      // TB >= 1.0+
+      gMessageNotificationBar.clearMsgNotifications();
+    }
+    else if (typeof(SetUpRemoteContentBar) == "function") {
+      // TB >= 0.8
       SetUpRemoteContentBar(null);
-      
+    }
+
     for (index in gMessageListeners)
       gMessageListeners[index].onStartHeaders();
 
